import network, urequests
import time
import math
import json
import picographics
from pimoroni import RGBLED
from pimoroni import Button
import jpegdec
import secrets

class point:
    def __init__(self,lat,lon):
        self.lat = lat
        self.lon = lon

gs = point(secrets.LATITUDE, secrets.LONGITUDE)
iss = point(0,0)
display = picographics.PicoGraphics(picographics.DISPLAY_PICO_DISPLAY, pen_type=picographics.PEN_RGB332, rotate=0)
WHITE = display.create_pen(255,255,255)
BLACK = display.create_pen(0,0,0)
RED = display.create_pen(255,0,0)
led = RGBLED(6, 7, 8)
button_a = Button(12)
button_b = Button(13)
EARTHRADIUS = 6371e3


def clear():
    display.set_pen(BLACK)
    display.clear()
    display.update()

def initDisplay():
    clear()
    display.set_backlight(0.5)
    led.set_rgb(0, 0, 0)
    display.set_font("bitmap8")
    display.set_pen(WHITE)
    
def start():
    display.text("Starting", 10,10, scale=2)
    display.update()
    
def distance(p1, p2):
    # Calculated using the Haversine formula
    φ1 = p1.lat * math.pi/180 # φ, λ in radians
    φ2 = p2.lat * math.pi/180
    Δφ = (p2.lat - p1.lat) * math.pi/180
    Δλ = (p2.lon - p1.lon) * math.pi/180
    a = math.sin(Δφ/2) * math.sin(Δφ/2) + math.cos(φ1) * math.cos(φ2) * math.sin(Δλ/2) * math.sin(Δλ/2)
    c = 2 * math.atan2(math.sqrt(a), math.sqrt(1-a))

    d = EARTHRADIUS * c # in metres
    return d

def setLED(distance):
    distance = distance / 1000
    if distance <= 5000 and distance > 4000:
        led.set_rgb(127,0,0)
    elif distance <= 4000 and distance > 3000:
        led.set_rgb(255,0,0)
    elif distance <= 3000 and distance > 2000:
        led.set_rgb(0,127,0)
    elif distance <= 2000 and distance > 1000:
        led.set_rgb(0,255,0)
    elif distance <= 1000:
        led.set_rgb(255,255,255)
    else:
        led.set_rgb(0,0,0)

def convertToXY(p1):
    # Convert to radians
    x = int((240/360) * (180 + p1.lon))
    y = int((120/180) * (90 - p1.lat))
    return (x,y)

def getPos():
    req = urequests.get("http://api.open-notify.org/iss-now.json")
    obj = req.json()
    tmint = int(obj['timestamp'])
    tm = time.localtime(tmint)
    lat = float(obj['iss_position']['latitude'])
    lon = float(obj['iss_position']['longitude'])
    return (tm, lat, lon)

def showMap():
    clear()
    display.load_spritesheet("spriteSheet.rgb332")
    j = jpegdec.JPEG(display)
    j.open_file("world.jpg")
    buttonPressed = False
    while not buttonPressed:
        j.decode(0, 0, jpegdec.JPEG_SCALE_FULL)
        (tm,iss.lat,iss.lon) = getPos()
        d = distance(gs, iss)
        setLED(d)
        (px, py) = convertToXY(iss)
        cnt = 0
        while (cnt < 5) and (not buttonPressed):
            display.sprite(0,0,px,py)
            display.update()
            time.sleep(0.5)
            display.sprite(0,1,px,py)
            display.update()
            time.sleep(0.5)
            cnt += 1
            if button_b.read():
                buttonPressed = True
        
def showLatLon():
    buttonPressed = False
    while not buttonPressed:
        (tm, iss.lat, iss.lon) = getPos()
        d = distance(gs, iss)
        setLED(d)
        # a = azimuth(gs, iss)
        display.set_pen(BLACK)
        display.clear()
        display.update()
        display.set_pen(WHITE)
        # display.text(tmStr, 10, 10, scale=2)
        latStr = "Lat: %.4f" % (iss.lat)
        lonStr = "Lon: %.4f" % (iss.lon)
        dist = "Dist: %.2f" % (d/1000)
        display.text(latStr, 10, 20, scale=3)
        display.text(lonStr, 10, 50, scale=3)
        display.text(dist, 10, 80, scale=3)
        display.update()
        cnt = 0
        while (cnt < 5) and (not buttonPressed):
            dx = int(cnt * 240 / 5)
            time.sleep(1)
            cnt += 1
            display.line(dx, 110, int(cnt * 240 / 5), 110) 
            display.update()
            if button_a.read():
                buttonPressed = True



wlan = network.WLAN(network.STA_IF)
wlan.active(True)
wlan.connect(secrets.SSID, secrets.PASSWORD)
initDisplay()
start()

while not wlan.isconnected() and wlan.status() >= 0:
    display.text("Waiting to connect:", 0, 0, scale = 2)
    time.sleep(1)

print(wlan.ifconfig())
while True:
    showLatLon()
    idx = 0
    while idx < 5:
        time.sleep(1)
        if button_a.read():
            showMap()
            idx = 5
        idx += 1



